/*
 * include/asm-mips/types.h
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994, 1995 by Waldorf GMBH
 * written by Ralf Baechle
 */
#ifndef __ASM_MIPS_TYPES_H

/* Any one of these symbols __need_* means that GNU libc
   wants us just to define one data type.  So don't define
   the symbols that indicate this file's entire job has been done.  */
#if !defined(__need_time_t) && !defined(__need_clock_t)
#define __ASM_MIPS_TYPES_H
#endif

#ifdef __ASM_MIPS_TYPES_H

#define __need_size_t
#define __need_ptrdiff_t
#include <stddef.h>

/*
 * This one is missing in <stddef.h>
 */
#ifndef _SSIZE_T
#define _SSIZE_T
typedef __SSIZE_TYPE__ ssize_t;
#endif
#endif /* defined (__ASM_MIPS_TYPES_H) */

/*
 * Argl...  Some stupid software realy plays with the _TIME_T and _CLOCK_T
 * wrapper symbols.  So I keep 'em for now.
 */
#if !defined (__time_t_defined) && \
    (defined(__ASM_MIPS_TYPES_H) || defined (__need_time_t))
#define __time_t_defined
#ifndef _TIME_T
#define _TIME_T
typedef long time_t;
#endif
#endif
#undef __need_time_t

#if !defined (__clock_t_defined) && \
    (defined(__ASM_MIPS_TYPES_H) || defined (__need_clock_t))
#define __clock_t_defined
#ifndef _CLOCK_T
#define _CLOCK_T
typedef long clock_t;
#endif
#endif
#undef __need_clock_t

#ifdef __ASM_MIPS_TYPES_H

/*
 * __xx is ok: it doesn't pollute the POSIX namespace. Use these in the
 * header files exported to user space
 */

typedef __signed__ char __s8;
typedef unsigned char __u8;

typedef __signed__ short __s16;
typedef unsigned short __u16;

typedef __signed__ int __s32;
typedef unsigned int __u32;

#if ((~0UL) == 0xffffffff)

#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
typedef __signed__ long long __s64;
typedef unsigned long long __u64;
#endif
  
#else
  
typedef __signed__ long __s64;
typedef unsigned long __u64;

#endif

/*
 * These aren't exported outside the kernel to avoid name space clashes
 */
#ifdef __KERNEL__

typedef __signed char s8;
typedef unsigned char u8;

typedef __signed short s16;
typedef unsigned short u16;

typedef __signed int s32;
typedef unsigned int u32;

#if ((~0UL) == 0xffffffff)

#if defined(__GNUC__) && !defined(__STRICT_ANSI__)
typedef __signed__ long long s64;
typedef unsigned long long u64;
#endif
  
#else
  
typedef __signed__ long s64;
typedef unsigned long u64;

#endif

#endif /* __KERNEL__ */

typedef long pid_t;
typedef long uid_t;
typedef long gid_t;
typedef unsigned long dev_t;
typedef unsigned long ino_t;
typedef unsigned long mode_t;
typedef unsigned long umode_t;
typedef unsigned long nlink_t;
typedef long daddr_t;
typedef long off_t;

#ifdef __KERNEL__

#undef __FD_SET
static __inline__ void __FD_SET(unsigned long __fd, fd_set *__fdsetp)
{
	unsigned long __tmp = __fd / __NFDBITS;
	unsigned long __rem = __fd % __NFDBITS;
	__fdsetp->fds_bits[__tmp] |= (1UL<<__rem);
}

#undef __FD_CLR
static __inline__ void __FD_CLR(unsigned long __fd, fd_set *__fdsetp)
{
	unsigned long __tmp = __fd / __NFDBITS;
	unsigned long __rem = __fd % __NFDBITS;
	__fdsetp->fds_bits[__tmp] &= ~(1UL<<__rem);
}

#undef __FD_ISSET
static __inline__ int __FD_ISSET(unsigned long __fd, fd_set *__p)
{ 
	unsigned long __tmp = __fd / __NFDBITS;
	unsigned long __rem = __fd % __NFDBITS;
	return (__p->fds_bits[__tmp] & (1UL<<__rem)) != 0;
}

/*
 * This will unroll the loop for the normal constant case (8 ints,
 * for a 256-bit fd_set)
 */
#undef __FD_ZERO
static __inline__ void __FD_ZERO(fd_set *__p)
{
	unsigned int *__tmp = __p->fds_bits;
	int __i;

	if (__builtin_constant_p(__FDSET_INTS)) {
		switch (__FDSET_INTS) {
			case 8:
				__tmp[0] = 0; __tmp[1] = 0;
				__tmp[2] = 0; __tmp[3] = 0;
				__tmp[4] = 0; __tmp[5] = 0;
				__tmp[6] = 0; __tmp[7] = 0;
				return;
		}
	}
	__i = __FDSET_INTS;
	while (__i) {
		__i--;
		*__tmp = 0;
		__tmp++;
	}
}

#endif /* __KERNEL__ */

#endif /* defined (__ASM_MIPS_TYPES_H) */

#endif /* !defined (__ASM_MIPS_TYPES_H) */
